-- Captain's Kit v1.1
-- By AbortRetryFail

declare("Ckit", 
	{
		["gunners"] = {}, -- Keeps current state of turret gunners.
		["assigned"] = {}, -- Turret assignments
	})

-- Local variables
local holdfire = false
local hudchat = true
local gunners = Ckit.gunners
local shipstations = 
	{
		-- Only ship station types with docking bays need to go here.
		["Trident Type M"] = true,
	}

-- Local functions and shortcuts
local function shipchat(msg) return SendChat(msg, "STATION") end
local function psm(msg) return HUD.PrintSecondaryMsg(HUD, msg) end

local function iscapship() 
	-- This exists because GetCurrentStationType() does not return 1 for the capship pilot. Blah.
	local stype = GetStationType()
	return stype and shipstations[stype] or false
end

local function kick(charid)
	if charid ~= GetCharacterID() then -- Don't kick yourself
		Gunner.Kick(GetPlayerName(charid))
	end
end

local function dismiss(charid)
	if charid ~= GetCharacterID() then -- Don't dismiss yourself
		Gunner.Dismiss(GetPlayerName(charid))
	end
end

-- Event handlers
function Ckit:TURRET_OCCUPIED(event, turretindex, charid)
	gunners[turretindex] = charid
	if holdfire 
	or (self.assigned[charid] and not self.assigned[charid][turretindex])
	then 
		dismiss(charid) 
		return 
	end
end

function Ckit:TURRET_EMPTY(event, turretindex, charid)
	if gunners[turretindex] then
		gunners[turretindex] = nil
	end
end

function Ckit:CHAT_MSG_BARENTER(event, args)
	if hudchat and iscapship() then
		local charname = string.match(args.msg, "(.-) has entered the room.")
		if charname then psm(charname.." docked.") end
	end
end

function Ckit:CHAT_MSG_BARLEAVE(event, args)
	if hudchat and iscapship() then
		local charname = string.match(args.msg, "(.-) has left the room.")
		if charname then psm(charname.." undocked.") end
	end
end

function Ckit:CHAT_MSG_SERVER(event, args)
	if (not iscapship()) then return end
	-- For Self-Destruct warnings and gunner kicks.
	if args.msg == "Self-destruct sequence activated. Accelerate or fire a weapon to cancel." then
		-- Self destruct warnings
		local warn = "WARNING: Self-destruct sequence activated! All crew abandon ship!"
		psm(warn)
		shipchat(warn)
	elseif args.msg == "5 seconds to self-destruct." then 
		-- Kick out all docked ships
		for charid, _ in pairs(GetBarPatrons()) do
			kick(charid)
		end
	end
end

RegisterEvent(Ckit, "TURRET_OCCUPIED")
RegisterEvent(Ckit, "TURRET_EMPTY")
RegisterEvent(Ckit, "CHAT_MSG_BARENTER")
RegisterEvent(Ckit, "CHAT_MSG_BARLEAVE")
RegisterEvent(Ckit, "CHAT_MSG_SERVER")
--RegisterEvent(Ckit, "CHAT_MSG_SECTORD") --For output of /gunner list

-- User Commands
function Ckit.holdfire()
	local msg = ""
	holdfire = not holdfire
	if holdfire then
		msg = "All gunners, hold your fire."
		for turretindex, charid in pairs(gunners) do
			dismiss(charid)
		end
	else
		msg = "All gunners, report to battle stations."
	end
	shipchat(msg)
	psm(msg)
end

RegisterUserCommand("holdfire", Ckit.holdfire)

function Ckit.assign(data, args)
	local charname = table.remove(args, 1)
	local charid = GetCharacterIDByName(charname)
	if not charid then return end
	if #args > 0 then
		Ckit.assigned[charid] = {}
		for _, arg in pairs(args) do
			local turret = tonumber(arg)
			if turret then 
				Ckit.assigned[charid][turret] = true
			end
		end
	else
		Ckit.assigned[charid] = nil
	end

end

RegisterUserCommand("assign", Ckit.assign)
